@@ -3,9 +3,11 @@ from time import sleep
 from xml.dom import minidom
 
 from ..constants import (
+    _LOGGER,
     ATTR_ACTION,
     ATTR_CONTROL,
     ATTR_FLAG,
+    ATTR_ID,
     ATTR_INSTANCE,
     ATTR_NODE_DEF_ID,
     ATTR_PRECISION,
@@ -13,7 +15,11 @@ from ..constants import (
     EVENT_PROPS_IGNORED,
     INSTEON_RAMP_RATES,
     ISY_VALUE_UNKNOWN,
+    NC_NODE_ERROR,
+    NODE_CHANGED_ACTIONS,
+    PROP_COMMS_ERROR,
     PROP_RAMP_RATE,
+    PROP_STATUS,
     PROTO_INSTEON,
     PROTO_NODE_SERVER,
     PROTO_ZIGBEE,
@@ -32,7 +38,6 @@ from ..constants import (
     TAG_PRIMARY_NODE,
     TAG_TYPE,
     UOM_SECONDS,
-    URL_STATUS,
     XML_ERRORS,
     XML_PARSE_ERROR,
     XML_TRUE,
@@ -187,18 +192,27 @@ class Nodes:
     def update_received(self, xmldoc):
         """Update nodes from event stream message."""
         address = value_from_xml(xmldoc, TAG_NODE)
-        value = value_from_xml(xmldoc, ATTR_ACTION)
+
+        node = self.get_by_id(address)
+        if not node:
+            _LOGGER.debug(
+                "Received a node update for node %s but could not find a record of this "
+                "node. Please try restarting the module if the problem persists, this "
+                "may be due to a new node being added to the ISY since last restart.",
+                address,
+            )
+            return
+        value = value_from_xml(xmldoc, ATTR_ACTION, "")
         value = int(value) if value != "" else ISY_VALUE_UNKNOWN
         prec = attr_from_xml(xmldoc, ATTR_ACTION, ATTR_PRECISION, "0")
         uom = attr_from_xml(xmldoc, ATTR_ACTION, ATTR_UNIT_OF_MEASURE, "")
-        node = self.get_by_id(address)
-        if not node:
-            return
-        # Check if UOM/PREC have changed or were not set:
-        node.update_precision(prec)
-        node.update_uom(uom)
-        node.status = value
-        self.isy.log.debug("ISY Updated Node: " + address)
+        formatted = value_from_xml(xmldoc, TAG_FORMATTED)
+
+        # Process the action and value if provided in event data.
+        node.update_state(
+            NodeProperty(PROP_STATUS, value, prec, uom, formatted, address)
+        )
+        _LOGGER.debug("ISY Updated Node: " + address)
 
     def control_message_received(self, xmldoc):
         """
@@ -212,16 +226,9 @@ class Nodes:
             # If there is no node associated with the control message ignore it
             return
 
-        # Process the action and value if provided in event data.
-        value = value_from_xml(xmldoc, ATTR_ACTION, 0)
-        value = int(value) if value != "" else ISY_VALUE_UNKNOWN
-        prec = attr_from_xml(xmldoc, ATTR_ACTION, ATTR_PRECISION, "0")
-        uom = attr_from_xml(xmldoc, ATTR_ACTION, ATTR_UNIT_OF_MEASURE, "")
-        formatted = value_from_xml(xmldoc, TAG_FORMATTED)
-
         node = self.get_by_id(address)
         if not node:
-            self.isy.log.debug(
+            _LOGGER.debug(
                 "Received a node update for node %s but could not find a record of this "
                 "node. Please try restarting the module if the problem persists, this "
                 "may be due to a new node being added to the ISY since last restart.",
@@ -229,14 +236,39 @@ class Nodes:
             )
             return
 
+        # Process the action and value if provided in event data.
+        node.update_last_update()
+        value = value_from_xml(xmldoc, ATTR_ACTION, 0)
+        value = int(value) if value != "" else ISY_VALUE_UNKNOWN
+        prec = attr_from_xml(xmldoc, ATTR_ACTION, ATTR_PRECISION, "0")
+        uom = attr_from_xml(xmldoc, ATTR_ACTION, ATTR_UNIT_OF_MEASURE, "")
+        formatted = value_from_xml(xmldoc, TAG_FORMATTED)
+
         if cntrl == PROP_RAMP_RATE:
             value = INSTEON_RAMP_RATES.get(value, value)
             uom = UOM_SECONDS
-        node_property = NodeProperty(cntrl, value, prec, uom, formatted)
-        if cntrl not in EVENT_PROPS_IGNORED:
-            node.aux_properties[cntrl] = node_property
+        node_property = NodeProperty(cntrl, value, prec, uom, formatted, address)
+        if (
+            cntrl == PROP_COMMS_ERROR
+            and value == 0
+            and PROP_COMMS_ERROR in node.aux_properties
+        ):
+            # Clear a previous comms error
+            del node.aux_properties[PROP_COMMS_ERROR]
+        elif cntrl not in EVENT_PROPS_IGNORED:
+            node.update_property(node_property)
         node.control_events.notify(node_property)
-        self.isy.log.debug("ISY Node Control Event: %s %s", address, node_property)
+        _LOGGER.debug("ISY Node Control Event: %s", node_property)
+
+    def node_changed_received(self, xmldoc):
+        """Handle Node Change/Update events from an event stream message."""
+        action = value_from_xml(xmldoc, ATTR_ACTION)
+        if not action or action not in NODE_CHANGED_ACTIONS:
+            return
+        node = value_from_xml(xmldoc, TAG_NODE)
+        if action == NC_NODE_ERROR:
+            _LOGGER.warning("ISY Could not communicate with device: %s", node)
+        # FUTURE: Handle additional node change actions to force updates.
 
     def parse(self, xml):
         """
@@ -247,7 +279,7 @@ class Nodes:
         try:
             xmldoc = minidom.parseString(xml)
         except XML_ERRORS:
-            self.isy.log.error("%s: Nodes", XML_PARSE_ERROR)
+            _LOGGER.error("%s: Nodes", XML_PARSE_ERROR)
             return False
 
         # get nodes
@@ -322,9 +354,7 @@ class Nodes:
                     # the ISY MAC address in newer versions of
                     # ISY firmwares > 5.0.6+ ..
                     if int(flag) & 0x08:
-                        self.isy.log.debug(
-                            "Skipping root group flag=%s %s", flag, address
-                        )
+                        _LOGGER.debug("Skipping root group flag=%s %s", flag, address)
                         continue
                     mems = feature.getElementsByTagName(TAG_LINK)
                     # Build list of members
@@ -349,20 +379,54 @@ class Nodes:
                         ),
                         ntype,
                     )
-            self.isy.log.debug("ISY Loaded %s", ntype)
+            _LOGGER.debug("ISY Loaded %s", ntype)
 
     def update(self, wait_time=0):
         """
-        Update the contents of the class.
+        Update the status and properties of the nodes in the class.
+
+        This calls the "/rest/status" endpoint.
 
         |  wait_time: [optional] Amount of seconds to wait before updating
         """
-        sleep(wait_time)
-        xml = self.isy.conn.request(self.isy.conn.compile_url([URL_STATUS]))
-        if xml is not None:
-            self.parse(xml)
-        else:
-            self.isy.log.warning("ISY Failed to update nodes.")
+        if wait_time:
+            sleep(wait_time)
+        xml = self.isy.conn.get_status()
+
+        if xml is None:
+            _LOGGER.warning("ISY Failed to update nodes.")
+            return
+
+        try:
+            xmldoc = minidom.parseString(xml)
+        except XML_ERRORS:
+            _LOGGER.error("%s: Nodes", XML_PARSE_ERROR)
+            return False
+
+        for feature in xmldoc.getElementsByTagName(TAG_NODE):
+            address = feature.attributes[ATTR_ID].value
+
+            if address in self.addresses:
+                self.get_by_id(address).update(xmldoc=feature)
+                continue
+
+        _LOGGER.info("ISY Updated Node Statuses.")
+
+    def update_nodes(self, wait_time=0):
+        """
+        Update the contents of the class.
+
+        This calls the "/rest/nodes" endpoint.
+
+        |  wait_time: [optional] Amount of seconds to wait before updating
+        """
+        if wait_time:
+            sleep(wait_time)
+        xml = self.isy.conn.get_nodes()
+        if xml is None:
+            _LOGGER.warning("ISY Failed to update nodes.")
+            return
+        self.parse(xml)
 
     def insert(self, address, nname, nparent, nobj, ntype):
         """
@@ -455,6 +519,17 @@ class Nodes:
             self.ntypes,
         )
 
+    def get_folder(self, address):
+        """Return the folder of a given node address."""
+        parent = self.nparents[self.addresses.index(address)]
+        if parent is None:
+            # Node is in the root folder.
+            return None
+        parent_index = self.addresses.index(parent)
+        if self.ntypes[parent_index] != TAG_FOLDER:
+            return self.get_folder(parent)
+        return self.nnames[parent_index]
+
     @property
     def children(self):
         """Return the children of the class."""
